/*
 * Copyright (C) 2016 favdb
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package assistant.app;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import storybook.toolkit.LOG;
import storybook.toolkit.file.EnvUtil;

/**
 *
 * @author favdb
 */
public class AssistantPref {

	private static final String TT = "Pref";

	List<PrefValue> preferences = new ArrayList<>();
	private final String prefFile = EnvUtil.getPrefFile().getAbsolutePath();
	public List<PrefFile> recentFiles = new ArrayList<>();

	public enum KEY {
		ASSISTANT("Assistant", ""),
		CONFIRM_EXIT("ConfirmExit", "1"),
		DATEFORMAT("DateFormat", "MM-dd-yyyy"),
		FILE_LAST("LastOpenFile", ""),
		FIRST_START("FirstStart", "1"),
		FONT_DEFAULT("FontDefault", AssistantConst.FONT_DEFAULT),
		ICON_SIZE("IconSize", "0"),
		ICON_SCREEN("IconScreen", "0"),
		IMAGE_LATSDIR("LastImageDir", ""),
		LAF("LookAndFeel", AssistantConst.LookAndFeel.LIGHT.name()),
		LANGUAGE("Language", "en_US"),
		LAYOUT_LAST_USED("LastUsedLayout", ""),
		MSGFILE("MessagesFile", ""),
		RECENT_FILES("RecentFiles", ""),
		TOOLBAR_ORIENTATION("ToolBarOrientation", "North"),
		UPDATER_DO("UpdaterDo", "0"),
		UPDATER_LAST("UpdaterLast", ""),
		VERSION("Version", AssistantConst.VERSION),
		WINDOW("Window", "0,100,100,800,600"),
		NONE("None", "");
		final private String text, value;

		private KEY(String text, String value) {
			this.text = text;
			this.value = value;
		}

		public String getValue() {
			return (value);
		}

		public Integer getInteger() {
			return (Integer.parseInt(value));
		}

		public boolean getBoolean() {
			return (value.equals("1") || value.equals("true"));
		}

		@Override
		public String toString() {
			return text;
		}
	}

	public AssistantPref() {
		load();
		String av = get(KEY.VERSION.toString(), KEY.VERSION.getValue());
		String cv = KEY.VERSION.getValue();
		if (!av.equals(cv)) {
			setString(KEY.VERSION.toString(), cv);
			LOG.log("Set Preferences version to " + cv);
		}
	}

	public String get(String key, String def) {
		return (getString(key, def));
	}

	public String get(KEY key, String def) {
		return (getString(key, def));
	}

	public String getString(KEY key) {
		return (getString(key.toString(), key.getValue()));
	}

	public String getString(KEY key, String def) {
		return (getString(key.toString(), def));
	}

	public String getString(String key, String def) {
		for (PrefValue v : preferences) {
			if (v.key.equals(key)) {
				return (v.value);
			}
		}
		if (!"".equals(def)) {
			preferences.add(new PrefValue(key, def));
		}
		return (def);
	}

	public String getChar(KEY key) {
		String r = getString(key, key.getValue());
		if (r.length() > 1) {
			int n = Integer.parseInt(r);
			String s = "" + (char) n;
			return (s);
		}
		return (r);
	}

	public Integer getInteger(KEY key) {
		if (key.getValue().isEmpty()) {
			return 0;
		}
		return getInteger(key, Integer.parseInt(key.getValue()));
	}

	public Integer getInteger(KEY key, Integer val) {
		String r = getString(key, val.toString());
		return (r.isEmpty() ? 0 : Integer.parseInt(r));
	}

	public boolean getBoolean(KEY key) {
		String b = key.getValue();
		return getBoolean(key.toString(), ("1".equals(b) || "true".equals(b)));
	}

	public boolean getBoolean(KEY key, boolean b) {
		return getBoolean(key.toString(), b);
	}

	public boolean getBoolean(String key, boolean val) {
		String r = getString(key, (val ? "true" : "false"));
		return "true".equals(r) || "1".equals(r);
	}

	public void setDate(KEY key, Date date) {
		if (date == null) {
			setString(key, "");
		} else {
			SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
			setString(key, fmt.format(date));
		}
	}

	public Date getDate(KEY key) {
		Date date = new Date();
		String str = getString(key);
		if (!str.isEmpty()) {
			try {
				SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
				date = fmt.parse(str);
			} catch (ParseException ex) {
				date = new Date();
			}
		}
		return date;
	}

	public void recentFilesAdd(String file, String title) {
		for (int i = 0; i < recentFiles.size(); i++) {
			PrefFile pf = recentFiles.get(i);
			if (pf.file.equals(file)) {
				recentFiles.remove(i);
				break;
			}
		}
		recentFiles.add(0, new PrefFile(file, title));
		recentFilesSave();
	}

	public void recentFilesClear() {
		recentFiles = new ArrayList<>();
		setString(KEY.RECENT_FILES, "");
	}

	public void recentFilesLoad() {
		recentFiles = new ArrayList<>();
		String v = getString(KEY.RECENT_FILES, "");
		if (!v.isEmpty()) {
			String vfiles[] = v.split(";");
			for (String vfile : vfiles) {
				String vx[] = vfile.split("\\|");
				if (vx.length == 2) {
					File f = new File(vx[0]);
					if (!f.exists()) {
						continue;
					}
					recentFiles.add(new PrefFile(vx[0], vx[1]));
				}
			}
		}
	}

	public void recentFilesSave() {
		StringBuilder b = new StringBuilder();
		recentFiles.forEach(pf -> {
			if (!b.toString().isEmpty()) {
				b.append(";");
			}
			File f = new File(pf.file);
			if (f.exists()) {
				b.append(pf.toString());
			}
		});
		setString(KEY.RECENT_FILES, b.toString());
	}

	public void set(String key, String value) {
		setString(key, value);
	}

	public void removeString(KEY key) {
		removeString(key.toString());
	}

	public void removeString(String keyName) {
		for (int i = 0; i < preferences.size(); i++) {
			if (preferences.get(i).key.equals(keyName)) {
				preferences.remove(i);
				return;
			}
		}
	}

	public void setString(KEY key, String value) {
		setString(key.toString(), value);
	}

	public void setString(String key, String value) {
		boolean notok = true;
		if (value == null || value.isEmpty() || value.equals("null")) {
			removeString(key);
			return;
		}
		for (PrefValue v : preferences) {
			if (v.key.equals(key)) {
				v.set(value);
				notok = false;
			}
		}
		if (notok) {
			preferences.add(new PrefValue(key, value));
		}
	}

	public void setChar(KEY key, String value) {
		int n = value.charAt(0);
		String f = String.format("%d", n);
		setString(key, f);
	}

	public void setInteger(KEY key, Integer value) {
		setString(key, value.toString());
	}

	public void setInteger(String key, Integer value) {
		setString(key, value.toString());
	}

	public void setBoolean(KEY key, boolean value) {
		setString(key, (value ? "true" : "false"));
	}

	public void setBoolean(String key, boolean value) {
		setString(key, (value ? "true" : "false"));
	}

	private boolean isToLoad(String key) {
		//App.trace(TT + ".isLoad(key=" + key + ")");
		if (key.startsWith("Editor_") || key.startsWith("Table_")) {
			return true;
		}
		for (KEY k : KEY.values()) {
			if (k.toString().equals(key)) {
				return (true);
			}
		}
		return (false);
	}

	public void load() {
		File file = new File(prefFile);
		if (!file.exists()) {
			create(file);
			return;
		}
		LOG.log("Load Preferences from " + file.getAbsolutePath());
		try {
			InputStream ips = new FileInputStream(prefFile);
			InputStreamReader ipsr = new InputStreamReader(ips);
			try ( BufferedReader br = new BufferedReader(ipsr)) {
				String ligne;
				while ((ligne = br.readLine()) != null) {
					String[] s = ligne.split("=");
					if (s.length < 2) {
						preferences.add(new PrefValue(s[0], ""));
					} else {
						if (isToLoad(s[0])) {
							preferences.add(new PrefValue(s[0], s[1]));
						}
					}
				}
				br.close();
			}
			recentFilesLoad();
		} catch (IOException e) {
			e.printStackTrace(System.err);
		}
	}

	// suite
	private void create(File file) {
		LOG.log("Create new Preferences in " + file.getAbsolutePath());
		try {
			EnvUtil.getPrefDir().mkdir();
			file.createNewFile();
		} catch (IOException e) {
			LOG.err("Unable to create new file " + file.getAbsolutePath());
		}
		preferences.add(new PrefValue(KEY.VERSION));
		preferences.add(new PrefValue(KEY.FONT_DEFAULT));
		save();
	}

	public void save() {
		preferences.sort((PrefValue c1, PrefValue c2) -> c1.key.compareTo(c2.key));
		String newline = System.getProperty("line.separator");
		try ( OutputStream f = new FileOutputStream(prefFile)) {
			for (PrefValue p : preferences) {
				if (isToLoad(p.key)) {
					f.write((p.toString() + newline).getBytes());
				}
			}
		} catch (FileNotFoundException ex) {
			LOG.err("Unable to save Preferences", ex);
		} catch (IOException ex) {
			LOG.err("Unable to save Preferences", ex);
		}
	}

	private static class PrefValue {

		String key;
		String value;

		public PrefValue(String k, String v) {
			key = k;
			value = v;
		}

		private PrefValue(KEY key) {
			this.key = key.toString();
			this.value = key.getValue();
		}

		public void set(String v) {
			value = v;
		}

		public String get() {
			return (value);
		}

		@Override
		public String toString() {
			return (key + "=" + value);
		}
	}

	public static class PrefFile {

		public String file, title;

		public PrefFile(String file, String title) {
			this.file = file;
			this.title = title;
		}

		@Override
		public String toString() {
			return (file + "|" + title);
		}
	}

}
